// ---------------------------------------------------------------------
/*
 Copyright (c) 2004-2006 Honda Research Institute Japan, Co., Ltd.

 \file	GHDSS_MathMat2.h
 \brief	\~English Header file for matrix operation. (GHDSS System)
 \brief	\~Japanese s񉉎Ẑ߂̃wb_t@C(GHDSS System)
 \author	Sachiko Suzuki
 \date	23 Oct 2007
 \version	$Id$
*/

// ---------------------------------------------------------------------
#ifndef GHDSS_MathMat2_h_defined
#define GHDSS_MathMat2_h_defined
// ---------------------------------------------------------------------
#include "MICARY_struct.h"
#include "MICARY_GHDSS_struct.h"

// ---------------------------------------------------------------------

/*
 \brief \~English It products complex matrix.
 \brief \~Japanese fs̊|Z
 \param pcaOutMat \~English It's output matrix struct.
 \~Japanese o͕fs
 \param caInMat1 \~English It's the complex matrix struct.
 \~Japanese ͕fs1
 \param caInMat2 \~English It's the complex matrix struct.
 \~Japanese ͕fs2
*/
int ProductCompMat(MICARY_ComplexArray2 *pcaOutMat,MICARY_ComplexArray2 caInMat1,
								 MICARY_ComplexArray2 caInMat2) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It products float matrix.
 \brief \~Japanese s̊|Z
 \param pcaOutMat \~English It's the float matrix struct.
 \~Japanese o͍s
 \param caInMat1 \~English It's the float matrix struct.
 \~Japanese ͍s1
 \param caInMat2 \~English It's the float matrix struct.
 \~Japanese ͍s2
*/
int ProductFloatMat(MICARY_FloatArray2 *pcaOutMat,MICARY_FloatArray2 caInMat1,
								MICARY_FloatArray2 caInMat2);

// ---------------------------------------------------------------------
/*
 \brief \~English It conjugattes complex matrix.(0:false 1:true)
 \brief \~Japanese 𕡑f(0:G[@1:I)
 \param pcaOutMat \~English It's the complex matrix struct.
 \~Japanese o͕fs
 \param caInMat \~English It's the complex matrix struct.
 \~Japanese ͕fs
*/
int ConjCompMat(MICARY_ComplexArray2 *pcaOutMat ,MICARY_ComplexArray2 caInMat);

// ---------------------------------------------------------------------
/*
 \brief \~English It makes diagonal complex matrix.
 \brief \~Japanese Ίps(s̏ꍇ)
 \param pcaOutMat \~English It's the complex matrix struct.
 \~Japanese o͕fs
   \param caInMat \~English It's the complex matrix struct.
 \~Japanese ͕fs
*/
int DiagCompMat(MICARY_ComplexArray2 *pcaOutMat,MICARY_ComplexArray2 caInMat) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It makes diagonal float matrix.
 \brief \~Japanese Ίps(s̏ꍇ)
 \param pcaOutMat \~English It's the float matrix struct.
 \~Japanese o͍s
 \param caInMat \~English It's the float matrix struct.
 \~Japanese ͍s
*/
int DiagFloatMat(MICARY_FloatArray2 *pcaOutMat,MICARY_FloatArray2 caInMat) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It products complex matrix.
 \brief \~Japanese fs̊|Z
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.
 \~Japanese ͕fs2
 \param g1 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r1 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
 
*/
int ProductCompMat1(MICARY_ComplexArray *pcaOutMat,MICARY_ComplexArray caInMat1,int g1,int r1,
									MICARY_ComplexArray caInMat2,int g2,int r2) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It products float matrix.
 \brief \~Japanese s̊|Z
 \param pcaOutMat \~English It's the float array struct.
 \~Japanese o͍s
 \param caInMat1 \~English It's the float array struct.
 \~Japanese ͍s1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͍s1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͍s1̗TCY
 \param caInMat2 \~English It's the float array struct.
 \~Japanese ͍s2
 \param g1 \~English It's the number of row for input matrix2.
 \~Japanese ͍s2̍sTCY
 \param r1 \~English It's the number of column for input matrix2.
 \~Japanese ͍s2̗TCY
 
*/
int ProductFloatMat1(MICARY_FloatArray *pcaOutMat ,MICARY_FloatArray caInMat1,int g1,int r1,
								MICARY_FloatArray caInMat2,int g2,int r2) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It conjugates complex matrix.
 \brief \~Japanese 𕡑f
 \param pcaOutMat \~English It's the complex matrix struct.
 \~Japanese o͕fs
 \param caInMat \~English It's the complex matrix struct.
 \~Japanese ͕fs
*/
int ConjCompMat1(MICARY_ComplexArray *pcaOutMat,MICARY_ComplexArray caInMat);

// ---------------------------------------------------------------------
/*
 \brief \~English It makes diagonal complex matrix.
 \brief \~Japanese Ίps(s̏ꍇ)
 \param pcaOutMat \~English It's the complex matrix struct.
 \~Japanese o͕fs
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͕fs
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/
int DiagCompMat1(MICARY_ComplexArray *pcaOutMat ,MICARY_ComplexArray caInMat, int g , int r) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It makes diagonal float matrix.
 \brief \~Japanese Ίps(s̏ꍇ)
 \param pcaOutMat \~English It's the float matrix struct.
 \~Japanese o͍s
   \param caInMat \~English It's the float matrix struct.
 \~Japanese ͍s
*/
int DiagFloatMat1(MICARY_FloatArray *pcaOutMat ,MICARY_FloatArray caInMat,int g,int r);
// ---------------------------------------------------------------------
/*
 \brief \~English It translates conjugate complex matrix.
 \brief \~Japanese ]uf
 \param pcaOutMat \~English It's the complex matrix struct.
 \~Japanese o͕fs
 \param caInMat \~English It's the complex matrix struct.
 \~Japanese ͕fs
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/
int ConjTransCompMat1(MICARY_ComplexArray *pcaOutMat ,MICARY_ComplexArray caInMat,int g,int r) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It makes unit matrix.
 \brief \~Japanese Pʍs̍쐬
 \param g \~English It's the complex matrix struct
 \~Japanese o͕fs
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/
int EyeCompMat1(MICARY_ComplexArray *pcaOutMat,int g,int r);

int EyeCompMat1v2(MICARY_ComplexArray *caOutMat,int g,int r) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It transrates complex matrix.
 \brief \~Japanese f̓]u
 \param pcaOutMat \~English It's the complex matrix struct.
 \~Japanese o͕fs
 \param caInMat \~English It's the complex matrix struct.
 \~Japanese ͕fs
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/

int TransCompMat1(MICARY_ComplexArray *pcaOutMat ,MICARY_ComplexArray caInMat,int g,int r) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It adds complex matrix.
 \brief \~Japanese fŝZ
 \param pcaOutMat1 \~English It's the complex array struct.
 \~Japanese o͕fs1
 \param caInMat1 \~English It's the complex array struct.
 \~Japanese ͕fs1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.
 \~Japanese ͕fs2
 \param g1 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r1 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
 
*/

int SumCompMat1(MICARY_ComplexArray *pcaOutMat ,MICARY_ComplexArray caInMat1,int g1,int r1,
							 MICARY_ComplexArray caInMat2,int g2,int r2) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It adds float matrix.
 \brief \~Japanese ŝZ
 \param pcaOutMat \~English It's the float array struct.
 \~Japanese o͍s
 \param caInMat1 \~English It's the float array struct.
 \~Japanese ͍s1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͍s1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͍s1̗TCY
 \param caInMat2 \~English It's the float array struct.
 \~Japanese ͍s2
 \param g1 \~English It's the number of row for input matrix2.
 \~Japanese ͍s2̍sTCY
 \param r1 \~English It's the number of column for input matrix2.
 \~Japanese ͍s2̗TCY
 
*/

int SumFloatMat1(MICARY_FloatArray *pcaOutMat ,MICARY_FloatArray caInMat1,int g1,int r1,
									MICARY_FloatArray caInMat2,int g2,int r2) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It substracts complex matrix.
 \brief \~Japanese fŝЂZ
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param caInMat1 \~English It's the complex array struct.
 \~Japanese ͕fs1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͕fs1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͕fs1̗TCY
 \param caInMat2 \~English It's the complex array struct.
 \~Japanese ͕fs2
 \param g1 \~English It's the number of row for input matrix2.
 \~Japanese ͕fs2̍sTCY
 \param r1 \~English It's the number of column for input matrix2.
 \~Japanese ͕fs2̗TCY
 
*/

int SubCompMat1(MICARY_ComplexArray *pcaOutMat,MICARY_ComplexArray caInMat1,int g1,int r1,
							 MICARY_ComplexArray caInMat2,int g2,int r2) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It subtracts float matrix.
 \brief \~Japanese ŝЂZ
 \param pcaOutMat \~English It's the float array struct.
 \~Japanese o͍s
 \param caInMat1 \~English It's the float array struct.
 \~Japanese ͍s1
 \param g1 \~English It's the number of row for input matrix1.
 \~Japanese ͍s1̍sTCY
 \param r1 \~English It's the number of column for input matrix1.
 \~Japanese ͍s1̗TCY
 \param caInMat2 \~English It's the float array struct.
 \~Japanese ͍s2
 \param g1 \~English It's the number of row for input matrix2.
 \~Japanese ͍s2̍sTCY
 \param r1 \~English It's the number of column for input matrix2.
 \~Japanese ͍s2̗TCY
 
*/

int SubFloatMat1(MICARY_FloatArray *pcaOutMat ,MICARY_FloatArray caInMat1,int g1,int r1,
									MICARY_FloatArray caInMat2,int g2,int r2) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It multiplicates complex matrix.
 \brief \~Japanese fs̒萔{
 \param pcaOutMat \~English It's the complex array struct.
 \~Japanese o͕fs
 \param caInMat1 \~English It's the complex array struct.
 \~Japanese ͕fs
 \param a \~English It's the coeficient value.
 \~Japanese Wl
 
*/

int MultiplCompMat1(MICARY_ComplexArray *pcaOutMat ,MICARY_ComplexArray caInMat,float a) ;
// ---------------------------------------------------------------------
/*
 \brief \~English It multiplicates float matrix.
 \brief \~Japanese sz̒萔{
 \param pcaOutMat \~English It's the float array struct.
 \~Japanese o͍s
 \param caInMat \~English It's the float array struct.
 \~Japanese ͍s
 \param a \~English It's the coefficient value.
 \~Japanese Wl
 
*/

int MultiplFloatMat1(MICARY_FloatArray *pcaOutMat ,MICARY_FloatArray caInMat,float a) ;

// ---------------------------------------------------------------------
/*
 \brief \~English It makes unit float matrix.
 \brief \~Japanese Pʍs̍쐬
 \param pcaOutMat \~English It's the complex  matrix.
 \~Japanese o͕fs
 \param g \~English It's the number of row for input matrix.
 \~Japanese ͍s̍sTCY
 \param r \~English It's the number of column for input matrix.
 \~Japanese ͍s̗TCY
*/
int EyeFloatMat1(MICARY_FloatArray *pcaOutMat ,int g,int r) ;

// ---------------------------------------------------------------------
/*
 \brief \~English It computes a dot product of a conjugated vector with another vector(0:false 1:true).
 \brief \~Japanese dot|Z(0:G[ 1:I)
 \param pcaOut \~English It's the complex  vector.
 \~Japanese o͕fxNg
 \param caIn1 \~English It's the input vector1.
 \~Japanese ͕fxNg1
 \param caIn2 \~English It's the input vector2.
 \~Japanese ͕fxNg2
*/
int DotcCompMat1(MICARY_ComplexArray *pcaOut ,MICARY_ComplexArray caIn1 ,MICARY_ComplexArray caIn2) ;

// ---------------------------------------------------------------------
/*
 \brief \~English It calculates abs complex matrix.
 \brief \~Japanese s̐ΒľvZ
 \param pcaOutMat \~English It's the complex  matrix.
 \~Japanese o͕fs
 \param caInMat \~English It's the complex  matrix.
 \~Japanese ͕fs
*/
int AbsCompMat1(MICARY_ComplexArray *pcaOutMat ,MICARY_ComplexArray caInMat) ;

// ---------------------------------------------------------------------
/*
 \brief \~English It calculates max value.
 \brief \~Japanese s(real part)̍őľvZ
 \param caInMat \~English It's the complex  matrix.
 \~Japanese ͕fs
*/
HA_Float MaxCompMat1(MICARY_ComplexArray caInMat) ;

// ---------------------------------------------------------------------

#endif


